#cs
	---------------------------------------------------
	ISN cmd (ISN AutoIt Studio Command Line Tool)
	---------------------------------------------------

	You can use this little command line toolkit to control the ISN AutoIt Studio via some cmd switches.
	For example with "ISN_cmd.exe /compile_project" you can start the compiling of the project.

	---------------------------------------------------
#ce

#NoTrayIcon

#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
	#AutoIt3Wrapper_Icon=../ISN AutoIt Studio/autoitstudioicon_red.ico
	#AutoIt3Wrapper_Res_Comment=https://www.isnetwork.at
	#AutoIt3Wrapper_Res_Description=ISN AutoIt Studio Command Line Tool
	#AutoIt3Wrapper_Res_Fileversion=0.0.2.0
	#AutoIt3Wrapper_Res_ProductVersion=0.0.2.0
	#AutoIt3Wrapper_Res_LegalCopyright=ISI360
	#AutoIt3Wrapper_Res_Language=1031
	#AutoIt3Wrapper_Res_Field=ProductName|ISN AutoIt Studio Command Line Tool
	#AutoIt3Wrapper_Run_Au3Stripper=y
	#Au3Stripper_Parameters=/mo
	#AutoIt3Wrapper_UseUpx=n
	#AutoIt3Wrapper_Run_Tidy=y
	#AutoIt3Wrapper_Res_HiDpi=Y
	#AutoIt3Wrapper_Change2CUI=y
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****

Global $Tool_Version = "0.02"
#include <WindowsConstants.au3>
#include <File.au3>
#include "..\ISN AutoIt Studio\Data\Plugins\PLUGIN SDK\isnautoitstudio_plugin.au3"

If Not IsArray($CmdLine) Then Exit
If $CmdLine[0] = 0 Then
	Exit ;No Parameters
EndIf

;Switches and settings
Global $ProcessCMDCommands = 1
Global $isn_start_allowed = "false"
Global $Autoit_Studio_exe_path = ""
Global $NewStudioPID = ""
Global $ISN_Startup_Timeout = 45
Global $ISN_error_behavior = 0
Global $ISN_AutoIt_Studio_Mainwindow_Handle = ""

$Dummy_GUI = GUICreate("ISN CMD Toolkit", 400, 100)
_ConsoleWrite("####################################################################" & @CRLF)
_ConsoleWrite("## ISN AutoIt Studio Command Line Tool v. " & $Tool_Version & " by ISI360" & @CRLF)
_ConsoleWrite("####################################################################" & @CRLF)

_ConsoleWrite(@CRLF)
_Setup_CMD()
_Process_CMD_Commands()


Func _Setup_CMD()
	;Setup CMD Commands
	If IsArray($CmdLine) Then
		For $x = 1 To $CmdLine[0]

			;First, the switches:
			If StringInStr($CmdLine[$x], "/isn_start_allowed") Then $isn_start_allowed = "true"
			If StringInStr($CmdLine[$x], "/isn_startup_timeout ") Then $ISN_Startup_Timeout = Number(StringStripWS(StringReplace($CmdLine[$x], "/isn_startup_timeout ", ""), 3))


			;/help OR /?
			If StringInStr($CmdLine[$x], "/help") Or StringInStr($CmdLine[$x], "/?") Then
				$helpstring = "You can use this little command line tool to control the ISN AutoIt Studio via some cmd switches." & @CRLF & _
						"NOTE: If you use switches with parameters, it is recommended to put the whole command into quotes!" & @CRLF & _
						'EXAMPLE: ISN_cmd.exe "/isn_start_allowed C:\ISN\AutoIt_Studio.exe" "/isn_open_file C:\temp\test.au3" "/isn_shutdown"' & @CRLF & @CRLF & _
						"Available commands:" & @CRLF & @CRLF & _
						"/?" & @TAB & @TAB & @TAB & @TAB & "This help text" & @CRLF & _
						"/isn_gui_handle" & @TAB & @TAB & @TAB & "Sets the ISN AutoIt Studio main gui handle. If this parameter is not available, the command line tool tries to find the handle automatically." & @CRLF & _
						"/isn_start_allowed" & @TAB & @TAB & "If this parameter is set and the command line tool finds no running ISN instance, it will automatically start a new one." & @CRLF & _
						@TAB & @TAB & @TAB & @TAB & "[optional] You can define the path to the AutoIt_Studio.exe as optional parameter. If its not defined, the command line tool tries to find the path by its own." & @CRLF & _
						"/isn_startup_timeout" & @TAB & @TAB & "If the command line tool starts a new ISN instance, you can overwrite the default 45 secounds timout for waiting for the new isn instance. (value is in secounds)" & @CRLF & _
						"/isn_compile_project" & @TAB & @TAB & "Compiles the currently opened project in the ISN AutoIt Studio." & @CRLF & _
						"/isn_close_project" & @TAB & @TAB & "Closes the currently opened project in the ISN AutoIt Studio." & @CRLF & _
						"/isn_backup_project" & @TAB & @TAB & "Execute the Backup function in the ISN AutoIt Studio." & @CRLF & _
						"/isn_test_project" & @TAB & @TAB & "Test (run) the currently opened project in the ISN AutoIt Studio." & @CRLF & _
						"/isn_restart" & @TAB & @TAB & @TAB & "Restarts the ISN AutoIt Studio immediately." & @CRLF & _
						"/isn_shutdown" & @TAB & @TAB & @TAB & "Exits the ISN AutoIt Studio immediately." & @CRLF & _
						"/isn_open_file" & @TAB & @TAB & @TAB & "Try to open a file in ISN AutoIt Studio. Must be a full path!" & @CRLF & _
						"/isn_open_project" & @TAB & @TAB & "Opens a ISN project in the ISN AutoIt Studio. Path must point to the *.isn file! Must be a full path!" & @CRLF & _
						"/isn_print_script" & @TAB & @TAB & "Displays the print preview of the currently opened file in the ISN AutoIt Studio!" & @CRLF & _
						"/isn_test_au3_file" & @TAB & @TAB & "Test (run) an *.au3 file within the ISN AutoIt Studio in the currently opened project. Must be a full path!" & @CRLF & _
						"/isn_error_behavior" & @TAB & @TAB & "Modifies the bahavior of the ISN AutoIt Studio when an error occurs. One of the following parameters is required:" & @CRLF & _
						@TAB & @TAB & @TAB & @TAB & "0 = Every error and MessageBox is displayed and requires user interaction to continue. (default)" & @CRLF & _
						@TAB & @TAB & @TAB & @TAB & "1 = If an error occurs, stop the processing of the command line tool and exit with error code <> 0. The ISN instance remains active." & @CRLF & _
						@TAB & @TAB & @TAB & @TAB & "2 = If an error occurs, stop the processing of the command line tool and exit with error code <> 0. The ISN instance will also exit immediately." & @CRLF & _
						@TAB & @TAB & @TAB & @TAB & "3 = Ignore all errors and continue processing." & @CRLF & _
						"/isn_trigger_macroslot" & @TAB & @TAB & "Runs the macro assigned to a specific macroslot (1-10) in the currently opened project." & @CRLF & _
						" "
				_ConsoleWrite("ISN AutoIt Studio Command Line Tool - Help:" & @CRLF & @CRLF)
				_ConsoleWrite($helpstring & @CRLF)
				Exit
			EndIf

			;/isn_gui_handle
			If StringInStr($CmdLine[$x], "/isn_gui_handle ") Then
				$GUI_Handle_to_Set = StringStripWS(StringReplace($CmdLine[$x], "/isn_gui_handle ", ""), 3)
				_ConsoleWrite("ISN main GUI handle set to " & $GUI_Handle_to_Set & "!" & @CRLF)
				$ISN_AutoIt_Studio_Mainwindow_Handle = Ptr($GUI_Handle_to_Set)
				$ISNPlugin_Status = "unlocked"
				$ISNPlugin_Message_Window_Handle = $Dummy_GUI
				GUIRegisterMsg(0x004A, "_ISNPlugin_Receive_Message") ;Register _WM_COPYDATA
			EndIf

			;/isn_start_allowed with aditional path
			If StringInStr($CmdLine[$x], "/isn_start_allowed ") Then
				Local $StudioPath_to_Set = StringStripWS(StringReplace($CmdLine[$x], "/isn_start_allowed ", ""), 3)
				If Not FileExists($StudioPath_to_Set) Then ContinueLoop
				_ConsoleWrite("Path to AutoIt_Studio.exe manually set to: " & $StudioPath_to_Set & "!" & @CRLF)
				$Autoit_Studio_exe_path = $StudioPath_to_Set
			EndIf

			;/isn_error_behavior
			If StringInStr($CmdLine[$x], "/isn_error_behavior ") Then
				$ISN_error_behavior = Number(StringStripWS(StringReplace($CmdLine[$x], "/isn_error_behavior ", ""), 3))
				_ConsoleWrite("ISN error behavior set to " & $ISN_error_behavior & "!" & @CRLF)
			EndIf

		Next
	EndIf
EndFunc   ;==>_Setup_CMD

Func _Process_CMD_Commands()

	;If /isn_gui_handle has not already set the needed $ISNPlugin_Message_Window_Handle value, we will find it...
	If $ISN_AutoIt_Studio_Mainwindow_Handle = "" Then
		_ConsoleWrite("ISN gui handle not set! Try to find the ISN AutoIt Studio main gui...")
		Opt("WinTitleMatchMode", 2)
		Opt("WinDetectHiddenText", 1)
		$Found_GUI = WinGetHandle("[TITLE:ISN AutoIt Studio; CLASS:AutoIt v3 GUI]", "ISN_MAIN_GUI")
		If @error Then
			_ConsoleWrite("ERROR: No ISN AutoIt Studio window found!" & @CRLF)
			If $isn_start_allowed = "false" Then
				_ConsoleWrite("/isn_start_allowed is NOT specified! So i am not allowed to start a new ISN instance." & @CRLF)
			Else
				_ConsoleWrite("/isn_start_allowed IS set! Lets start a new ISN instance..." & @CRLF)
				If $Autoit_Studio_exe_path = "" Then $Autoit_Studio_exe_path = _Find_Autoit_Studio_exe()
				If $Autoit_Studio_exe_path <> "" And FileExists($Autoit_Studio_exe_path) Then
					If $NewStudioPID = "" Then
						$NewStudioPID = Run($Autoit_Studio_exe_path)
						_ConsoleWrite("New ISN instance started from " & $Autoit_Studio_exe_path & " with PID " & $NewStudioPID & "!" & @CRLF)
					EndIf
					_ConsoleWrite("Waiting for the new ISN instance to become ready..." & @CRLF)
					Local $ISNWin_state
					Local $Timeout = $ISN_Startup_Timeout
					While 1
						$Found_GUI = WinGetHandle("[TITLE:ISN AutoIt Studio; CLASS:AutoIt v3 GUI]", "ISN_MAIN_GUI")
						If @error = 0 Then
							$ISNWin_state = WinGetState($Found_GUI, "")
							If BitAND($ISNWin_state, 2) Then ExitLoop
						EndIf
						Sleep(1000)
						$Timeout = $Timeout - 1
						If $Timeout < 1 Then
							_ConsoleWrite("ERROR: Timeout for ISN Startup expired!" & @CRLF)
							Return
						EndIf
					WEnd
					Sleep(1000)
					Return _Process_CMD_Commands() ;Start again
				Else
					_ConsoleWrite("ERROR: Cannot start new ISN instance! AutoIt_Studio.exe not found and no manually path is specified!" & @CRLF)
					Return
				EndIf
			EndIf
			Return
		Else
			Opt("WinTitleMatchMode", 1)
			Opt("WinDetectHiddenText", 0)
			$ISN_AutoIt_Studio_Mainwindow_Handle = Ptr($Found_GUI)
			$ISNPlugin_Status = "unlocked"
			$ISNPlugin_Message_Window_Handle = $Dummy_GUI
			GUIRegisterMsg(0x004A, "_ISNPlugin_Receive_Message") ;Register _WM_COPYDATA
			_ConsoleWrite("ISN AutoIt Studio found! (GUI Handle: " & $ISN_AutoIt_Studio_Mainwindow_Handle & ")" & @CRLF)
		EndIf
		Opt("WinTitleMatchMode", 1)
	EndIf


	;Fitst, update $ISN_error_behavior in the ISN
	_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_ISN_Set_Error_Behavior", String($ISN_error_behavior))



	;Main CMD Commands
	If IsArray($CmdLine) Then
		For $x = 1 To $CmdLine[0]

			If $ProcessCMDCommands <> 1 Then ExitLoop

			;/isn_open_project
			If StringInStr($CmdLine[$x], "/isn_open_project ") Then
				$File_to_open = StringStripWS(StringReplace($CmdLine[$x], "/isn_open_project ", ""), 3)
				If FileExists($File_to_open) And StringInStr($File_to_open, ".isn") Then
					_ConsoleWrite("Open project command send to the ISN AutoIt Studio!" & @CRLF)
					_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_Close_Project", "false")
					$File_to_open = StringTrimRight($File_to_open, StringLen($File_to_open) - StringInStr($File_to_open, "\", 0, -1) + 1)
					Local $LoadingGUI_Handle = HWnd(_ISNPlugin_Get_Variable_from_ISN_AutoIt_Studio("$Loading_GUI"))
					_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_Load_Project_by_Foldername", $File_to_open)
					_ConsoleWrite("Waiting until the project is loaded into the ISN session..." & @CRLF)
					_ConsoleWrite(@CRLF)
					Sleep(1500) ;Give the ISN some Time...
					;Wait while the compile GUI is active
					Local $Win_state
					While 1
						$Win_state = WinGetState($LoadingGUI_Handle, "")
						If Not BitAND($Win_state, 2) Then ExitLoop
						Sleep(250)
					WEnd
				Else
					_ConsoleWrite("File '" & $File_to_open & "' not found!" & @CRLF)
				EndIf
			EndIf

			;/isn_backup_project
			If StringInStr($CmdLine[$x], "/isn_backup_project") Then
				_ConsoleWrite("Backup command send to the ISN AutoIt Studio!" & @CRLF)
				_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_Backup_Files")
			EndIf

			;/isn_open_file
			If StringInStr($CmdLine[$x], "/isn_open_file ") Then
				$File_to_open = StringStripWS(StringReplace($CmdLine[$x], "/isn_open_file ", ""), 3)
				If FileExists($File_to_open) Then
					_ConsoleWrite("Open file command send to the ISN AutoIt Studio!" & @CRLF)
					_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("Try_to_opten_file", $File_to_open)
				Else
					_ConsoleWrite("File '" & $File_to_open & "' not found!" & @CRLF)
				EndIf
			EndIf

			;/isn_test_au3_file
			If StringInStr($CmdLine[$x], "/isn_test_au3_file ") Then
				$File_to_test = StringStripWS(StringReplace($CmdLine[$x], "/isn_test_au3_file ", ""), 3)
				If FileExists($File_to_test) And StringInStr($File_to_test, ".au3") Then
					_ConsoleWrite("Register this instance as thread in the ISN studio..." & @CRLF)
					_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_ISN_AutoIt_Studio_Register_CommandLineTool", $Dummy_GUI, @AutoItPID) ;Needed to get the StdOut Stream from the ISN
					Sleep(250)
					_ConsoleWrite("Test au3 file command send to the ISN AutoIt Studio!" & @CRLF)
					_ConsoleWrite("Waiting until the running script has exited..." & @CRLF)
					_ConsoleWrite(@CRLF)
					_ConsoleWrite(@CRLF)
					_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_Testscript", $File_to_test)
					Sleep(1000) ;Give ISN Time to Run the testthread
					$TestthreadPID = _ISNPlugin_Get_Variable_from_ISN_AutoIt_Studio("$ISN_Helper_Threads[0][1]")
					While 1

						If Not ProcessExists($TestthreadPID) Then ExitLoop
						Sleep(250)
					WEnd
					_ConsoleWrite(@CRLF)
					_ConsoleWrite(@CRLF)
					Sleep(1000) ;Give the ISN some Time...
					_ConsoleWrite("Unregister this instance as thread in the ISN studio..." & @CRLF)
					_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_ISN_AutoIt_Studio_Unregister_CommandLineTool")
				Else
					_ConsoleWrite("File '" & $File_to_test & "' not found or not an AutoIt 3 Script file (*.au3)!" & @CRLF)
				EndIf
			EndIf

			;/isn_compile_project
			If StringInStr($CmdLine[$x], "/isn_compile_project") Then
				_ConsoleWrite("Register this instance as thread in the ISN studio..." & @CRLF)
				_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_ISN_AutoIt_Studio_Register_CommandLineTool", $Dummy_GUI, @AutoItPID) ;Needed to get the StdOut Stream from the ISN
				Sleep(250)
				_ConsoleWrite("Compile command send to the ISN AutoIt Studio!" & @CRLF)
				$compilingGUI_Handle = _ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_Start_Compiling_Adlib")
				$compilingGUI_Handle = Ptr($compilingGUI_Handle)
				_ConsoleWrite("Waiting until the compile process is finished..." & @CRLF)
				_ConsoleWrite(@CRLF)

				Sleep(1500) ;Give the ISN some Time...
				;Wait while the compile GUI is active
				While 1
					$Win_state = WinGetState($compilingGUI_Handle, "")
					If Not BitAND($Win_state, 2) Then ExitLoop
					Sleep(250)
				WEnd
				_ConsoleWrite(@CRLF)
				_ConsoleWrite(@CRLF)
				Sleep(1000) ;Give the ISN some Time...
				_ConsoleWrite("Unregister this instance as thread in the ISN studio..." & @CRLF)
				_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_ISN_AutoIt_Studio_Unregister_CommandLineTool")

			EndIf

			;/isn_test_project
			If StringInStr($CmdLine[$x], "/isn_test_project") Then
				_ConsoleWrite("Register this instance as thread in the ISN studio..." & @CRLF)
				_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_ISN_AutoIt_Studio_Register_CommandLineTool", $Dummy_GUI, @AutoItPID) ;Needed to get the StdOut Stream from the ISN
				Sleep(250)
				_ConsoleWrite("Test project command send to the ISN AutoIt Studio!" & @CRLF)
				_ConsoleWrite("Waiting until the running script has exited..." & @CRLF)
				_ConsoleWrite(@CRLF)
				_ConsoleWrite(@CRLF)
				_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_Projekt_Testen")
				Sleep(1000) ;Give ISN Time to Run the testthread
				$TestthreadPID = _ISNPlugin_Get_Variable_from_ISN_AutoIt_Studio("$ISN_Helper_Threads[0][1]")
				While 1

					If Not ProcessExists($TestthreadPID) Then ExitLoop
					Sleep(250)
				WEnd
				_ConsoleWrite(@CRLF)
				_ConsoleWrite(@CRLF)
				Sleep(1000) ;Give the ISN some Time...
				_ConsoleWrite("Unregister this instance as thread in the ISN studio..." & @CRLF)
				_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_ISN_AutoIt_Studio_Unregister_CommandLineTool")

			EndIf

			;/isn_trigger_macroslot
			If StringInStr($CmdLine[$x], "/isn_trigger_macroslot ") Then
				Local $MacroslotNumber = Number(StringStripWS(StringReplace($CmdLine[$x], "/isn_trigger_macroslot ", ""), 3))
				If $MacroslotNumber = 0 Or $MacroslotNumber = "" Or Not IsNumber($MacroslotNumber) Then
					_ConsoleWrite("ERROR: No, or invalid macroslot ID specified!" & @CRLF)
					ContinueLoop
				EndIf

				_ConsoleWrite("Register this instance as thread in the ISN studio..." & @CRLF)
				_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_ISN_AutoIt_Studio_Register_CommandLineTool", $Dummy_GUI, @AutoItPID) ;Needed to get the StdOut Stream from the ISN
				Sleep(250)
				_ConsoleWrite("Trigger macroslot command send to the ISN AutoIt Studio!" & @CRLF)
				_ConsoleWrite("Waiting until the running macro has exited..." & @CRLF)
				_ConsoleWrite(@CRLF)
				_ConsoleWrite(@CRLF)
				Switch $MacroslotNumber

					Case 1
						_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_execute_macroslot_01")

					Case 2
						_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_execute_macroslot_02")

					Case 3
						_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_execute_macroslot_03")

					Case 4
						_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_execute_macroslot_04")

					Case 5
						_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_execute_macroslot_05")

					Case 6
						_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_execute_macroslot_06")

					Case 7
						_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_execute_macroslot_07")

					Case 8
						_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_execute_macroslot_08")

					Case 9
						_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_execute_macroslot_09")

					Case 10
						_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_execute_macroslot_10")


				EndSwitch
				Sleep(1000) ;Give ISN Time to Run the testthread
				Local $Regel_lauft = 1
				While 1

					$Regel_lauft = _ISNPlugin_Get_Variable_from_ISN_AutoIt_Studio("$Regel_lauft")
					If $Regel_lauft = 0 Then ExitLoop
					Sleep(2000)
				WEnd
				_ConsoleWrite(@CRLF)
				_ConsoleWrite(@CRLF)
				Sleep(1000) ;Give the ISN some Time...
				_ConsoleWrite("Unregister this instance as thread in the ISN studio..." & @CRLF)
				_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_ISN_AutoIt_Studio_Unregister_CommandLineTool")

			EndIf

			;/isn_print_script
			If StringInStr($CmdLine[$x], "/isn_print_script") Then
				_ConsoleWrite("Print command send to the ISN AutoIt Studio!" & @CRLF)
				_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_ISN_Print_current_file")
			EndIf


			;/close_project
			If StringInStr($CmdLine[$x], "/isn_close_project") Then
				_ConsoleWrite("Close project command send to the ISN AutoIt Studio!" & @CRLF)
				_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_Close_Project", "false")
			EndIf


			;/isn_shutdown
			If StringInStr($CmdLine[$x], "/isn_shutdown") Then
				_ConsoleWrite("Shutdown command send to the ISN AutoIt Studio!" & @CRLF)
				_ConsoleWrite("Closing opened projects..." & @CRLF)
				_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_Close_Project", "false")
				_ConsoleWrite("Shuting down the ISN Studio..." & @CRLF)
				_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_Shutdown_ISN_AutoIt_Studio")
				$ProcessCMDCommands = 0
				Return
			EndIf

			;/isn_restart
			If StringInStr($CmdLine[$x], "/isn_restart") Then
				_ConsoleWrite("Restart command send to the ISN AutoIt Studio!" & @CRLF)
				_ISNPlugin_Call_Async_Function_in_ISN_AutoIt_Studio("_Restart_ISN_AutoIt_Studio")
				$ProcessCMDCommands = 0
				Return
			EndIf

		Next
	EndIf


EndFunc   ;==>_Process_CMD_Commands

_Exit(0)

Func _Find_Autoit_Studio_exe()
	Local $Path = ""
	;Try in current location
	If FileExists(_PathFull("..\Autoit_Studio.exe", @WorkingDir)) Then Return _PathFull("..\Autoit_Studio.exe", @WorkingDir)

	;Try to find from installer location
	$RegRead = RegRead("HKLM\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\{7AAAE763-9EE4-448D-994F-94B52EEAA180}_is1", "InstallLocation")
	If @error Then $RegRead = RegRead("HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{7AAAE763-9EE4-448D-994F-94B52EEAA180}_is1", "InstallLocation")
	If Not @error And FileExists($RegRead & "Autoit_Studio.exe") Then Return $RegRead & "Autoit_Studio.exe"

	;Other typical paths
	If FileExists("C:\Program Files (x86)\ISN AutoIt Studio\Autoit_Studio.exe") Then Return "C:\Program Files (x86)\ISN AutoIt Studio\Autoit_Studio.exe"
	If FileExists("C:\Program Files\ISN AutoIt Studio\Autoit_Studio.exe") Then Return "C:\Program Files\ISN AutoIt Studio\Autoit_Studio.exe"
	If FileExists("D:\Program Files (x86)\ISN AutoIt Studio\Autoit_Studio.exe") Then Return "D:\Program Files (x86)\ISN AutoIt Studio\Autoit_Studio.exe"
	If FileExists("D:\Program Files\ISN AutoIt Studio\Autoit_Studio.exe") Then Return "D:\Program Files\ISN AutoIt Studio\Autoit_Studio.exe"
	If FileExists("C:\ISN AutoIt Studio\Autoit_Studio.exe") Then Return "C:\ISN AutoIt Studio\Autoit_Studio.exe"
	If FileExists("D:\ISN AutoIt Studio\Autoit_Studio.exe") Then Return "D:\ISN AutoIt Studio\Autoit_Studio.exe"


	Return "" ;Give up..
EndFunc   ;==>_Find_Autoit_Studio_exe

Func _ConsoleWrite($Message = "")
	ConsoleWrite($Message)
EndFunc   ;==>_ConsoleWrite

Func _Exit($ErrorCode = 0, $ErrorText = "")
	If $ErrorText <> "" Then _ConsoleWrite(@CRLF & $ErrorText & @CRLF & @CRLF)
	_ConsoleWrite(@CRLF & "ISN Command Line Tool will exit now with return code " & $ErrorCode & "..." & @CRLF & @CRLF)

	;Reset error behavior in ISN
	If $ProcessCMDCommands = 1 And $ISN_AutoIt_Studio_Mainwindow_Handle <> "" Then
		_ISNPlugin_Call_Function_in_ISN_AutoIt_Studio("_ISN_Set_Error_Behavior", String(0))
	EndIf

	Exit $ErrorCode
EndFunc   ;==>_Exit
